

def get_filename(name):

    return f'setup.cfg'



def get_content(name, directory=None):

    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir

    
    content = '''
    # Inside of setup.cfg
[metadata]
description-file = README.md

    '''
    return content




def get_filename(name):

    return f'README.md'


def get_content(name=None, directory=None):
    """
    """

    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir

    
    content = f'''
    # {name}
    <definition>


    ## How to use

    ```
    from {dir}{name} import {name}

    

    ```


    
    '''
    return content




def get_filename(name):

    return f'requirements.txt'


def get_content(name=None, directory=None):
    """
    """

    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir

    
    content = f'''
requests
kraken-thing
flask
gunicorn

    
    '''
    return content


import uuid
import datetime

def get_filename(name):

    return f'{name}/{name}.py'

def get_content(name, directory=None):
    """
    """
    class_name = name.replace('kraken_', '')
    
    class_name = class_name.capitalize()


    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir
    
    record_value = {}

    content = f'''
    
import copy
from {dir}{name}.helpers import json
from {dir}{name}.helpers import things
import os
import pkg_resources



"""
Notes:
To access files in data directory, use:
new_path = pkg_resources.resource_filename('{name}', old_path)

"""

        
def method1():
    """
    """
    
    return True


def method2():
    """
    """
    
    return True


def get_request_record(record):
    """Return the value from a request
        Handles, value, object or action record
    """
    if not isinstance(record, dict):
        return record

    if record.get("@type", None) == "action":
        record = record.get("object", None)

    return record
    

def get_instrument():
    """Returns an instrument record for this function
    """
    record = {{
        "@type": "WebAPI",
        "@id": "{str(uuid.uuid4())}",
        "name": "{name}"
    }}

    return record

def get_action(object, name, action=None):
    """Return an action record with instrument
    """

    # If action is provided, updates the info
    if action:
        action['instrument'] = get_instrument()
        action['timeStart'] = datetime.datetime.now()
        return action
        

    record = {{
        "@type": "action",
        "@id": str(uuid.uuid4()),
        "name": name,
        "timeStart": datetime.datetime.now(),
        "timeEnd": None,
        "object": object,
        "instrument": get_instrument(),
        "actionStatus": "potentialActionStatus"
    }}
    return record
    
    '''
    
    return content
    



import uuid


def get_filename(name):

    return f'{name}/helpers/json.py'


def get_content(name, directory=None):


    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir
    
    content = f'''
import datetime


def get_value(key, record):
    """Return value from different types of input records
    Accepts action record, or simple object
    """

    if record.get('@type', None) == 'action':
        record = record.get('object', None)

    value = record.get(key, None)

    return value

def get_instrument_record():
    """
    """
    record = {
        "@type": "WebAPI",
        "@id": "{str(uuid.uuid4())}",
        "name": "{name}",
        "description": "description"
    }

    return record

def get_action_record(object_record, result_record=None):
    """
    """

    action_record = {
        '@type': 'action',
        '@id': str(uuid.uuid4()),
        'name': '{name}',
        'object': object_record,
        'instrument': get_instrument(),
        'startTime': datetime.datetime.now(),
        'actionStatus': 'activeActionStatus'
    }

    if result_record:
        action_record['endTime] = datetime.datetime.now()
        action_record['actionStatus] = 'completedActionStatus'
        action_record['result] = result_record

    return action_record



    '''
    return content





import os
from kraken_init.helpers import template_class
from kraken_init.helpers import template_class_collection
from kraken_init.helpers import template_license_txt
from kraken_init.helpers import template_readme_md
from kraken_init.helpers import template_setup_cfg
from kraken_init.helpers import template_setup_py
from kraken_init.helpers import template_dockerfile
from kraken_init.helpers import template_tests
from kraken_init.helpers import template_git_ignore
from kraken_init.helpers import template_main
from kraken_init.helpers import template_requirements
from kraken_init.helpers import template_init
from kraken_init.helpers import template_helper_json
from kraken_init.helpers import template_helper_things
from kraken_init.helpers import template_flask
from kraken_init.helpers import template_methods





def init(name, force=True, directory=None):
    """
    Parameters:
        name: name of the package (kraken_something)
        force: repalces files if True
        directory: if specified, creates package in given directory instead of /
    """

    # Create init if directory
    if directory:
        d = []
        for i in directory.split('/'):
            d.append(i)
            filename = '__init__.py'
            write_to_file(filename, '', force, '/'.join(d))

    
    # Create __init__
    filename = f'{name}/helpers/__init__.py'
    content = ''
    write_to_file(filename, content, force, directory)

    # Create files
    filename = template_methods.get_filename(name)
    content = template_methods.get_content(name, directory)
    write_to_file(filename, content, force, directory)
    
    # Create files
    filename = template_class.get_filename(name)
    content = template_class.get_content(name, directory)
    write_to_file(filename, content, force, directory)

    # Create files
    filename = template_class_collection.get_filename(name)
    content = template_class_collection.get_content(name, directory)
    write_to_file(filename, content, force, directory)
    
    # Create files
    filename = template_license_txt.get_filename(name)
    content = template_license_txt.get_content(name, directory)
    write_to_file(filename, content, force, directory)
    
    # Create files
    filename = template_readme_md.get_filename(name)
    content = template_readme_md.get_content(name, directory)
    write_to_file(filename, content, force, directory)
    
    # Create files
    filename = template_setup_cfg.get_filename(name)
    content = template_setup_cfg.get_content(name, directory)
    write_to_file(filename, content, force, directory)
    
    # Create files
    filename = template_setup_py.get_filename(name)
    content = template_setup_py.get_content(name, directory)
    write_to_file(filename, content, force, directory)


    # Create files
    filename = template_dockerfile.get_filename(name)
    content = template_dockerfile.get_content(name, directory)
    write_to_file(filename, content, force, directory)

    # Create files
    filename = template_tests.get_filename(name)
    content = template_tests.get_content(name, directory)
    write_to_file(filename, content, force, directory)

    # Create files
    filename = template_git_ignore.get_filename(name)
    content = template_git_ignore.get_content(name, directory)
    write_to_file(filename, content, force, directory)

    # Create files
    filename = template_requirements.get_filename(name)
    content = template_requirements.get_content(name, directory)
    write_to_file(filename, content, force, directory)

    # Create files
    filename = template_init.get_filename(name)
    content = template_init.get_content(name, directory)
    write_to_file(filename, content, force, directory)

    # Create files
    filename = template_helper_json.get_filename(name)
    content = template_helper_json.get_content(name, directory)
    write_to_file(filename, content, force, directory)

    # Create files
    filename = template_helper_things.get_filename(name)
    content = template_helper_things.get_content(name, directory)
    write_to_file(filename, content, force, directory)
    
    # Create files
    filename = template_flask.get_filename(name)
    content = template_flask.get_content(name, directory)
    write_to_file(filename, content, force, directory)

    
    # Create main
    filename = template_main.get_filename(name)
    original_content = read_file(filename)
    new_content = template_main.get_content(name, directory)
    content = new_content + '\n' + original_content
    content = content.replace('kraken_init.init', '#kraken_init.init')
    write_to_file(filename, content, force, directory)

    return 


def write_to_file(filename, content, force=False, directory=None):
    """
    """

    if directory and directory != '': 
        filename = directory + '/' + filename


    
    
    try:
        os.makedirs(os.path.dirname(filename), exist_ok=True)
    except:
        a=1

    if os.path.exists(filename) and force == False:
        print(f'File {filename} already exists')
        return
        
    with open(filename, 'w', newline='') as f:
        f.write(content)
    return

def read_file(filename):
    """
    """

    with open(filename, 'r', newline='') as f:
        content = f.read()
    return content




def get_filename(name, directory=None):

    return f'Dockerfile'



def get_content(name, directory=None):

    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir
    
    content = f'''
    # Copyright 2020 Google, LLC.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# [START cloudrun_helloworld_dockerfile]
# [START run_helloworld_dockerfile]

# Use the official lightweight Python image.
# https://hub.docker.com/_/python
FROM python:3.10-slim

# Allow statements and log messages to immediately appear in the Knative logs
ENV PYTHONUNBUFFERED True

# Copy local code to the container image.
ENV APP_HOME /app
WORKDIR $APP_HOME
COPY . ./

# Install production dependencies.
RUN pip install --no-cache-dir -r requirements.txt

# Run the web service on container startup. Here we use the gunicorn
# webserver, with one worker process and 8 threads.
# For environments with multiple CPU cores, increase the number of workers
# to be equal to the cores available.
# Timeout is set to 0 to disable the timeouts of the workers to allow Cloud Run to handle instance scaling.
CMD exec gunicorn --bind :$PORT --workers 1 --threads 8 --timeout 0 {name}.flask_routes:app

# [END run_helloworld_dockerfile]
# [END cloudrun_helloworld_dockerfile]
    '''
    return content




def get_filename(name):

    return f'main.py'


def get_content(name, directory=None):
    """
    """
    
    class_name = name.replace('kraken_', '')
    class_name = class_name.capitalize()
    git_name = name.replace('_', '')
    class_name_collection = class_name + 's'


    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir
    
    content = f'''
import os
from {dir}{name} import {name}
from {dir}{name} import {class_name}
from {dir}{name} import {class_name_collection}
from {dir}{name} import flask_routes

"""
Project structure created by kraken_init

To dos
1. Configure pypi package publishing
- Todo: Configure github pypi publish action - DONE 
- Todo: Create pypi token (https://pypi.org/manage/account/)
- Todo: Add pypi api token to github secret (https://github.com/tactik8/{git_name}/settings/secrets/actions)
- Change pypi script "on" (trigger) to: 
    on:
      push:
        branches:
          - 'main'

2. Configure google cloud
- Todo: Create new cloud run service (https://console.cloud.google.com/run?project=kraken-v2-369412)

3. Add packages 
- Todo: Add required packages to setup.py
- Todo: Add required packages to requirements.txt
"""

def test():
    """Perform tests
    """
    os.system("pip install pytest")
    os.system("python -m pytest {name}* -vv")

# To run unit tests on package, uncomment the following line:
#test()

# To enable the flask api, uncomment the following line:
#flask_routes.run_api()

# To use classes, uncomment the following lines:
#{class_name.lower()} = {class_name}()
#{class_name_collection.lower()} = {class_name_collection}()

    '''
    return content




def get_filename(name):

    return f'.github/workflows/python-publish.yml'



def get_content(name, directory=None):

    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir
    
    content = '''
# This workflow will upload a Python Package using Twine when a release is created
# For more information see: https://docs.github.com/en/actions/automating-builds-and-tests/building-and-testing-python#publishing-to-package-registries

# This workflow uses actions that are not certified by GitHub.
# They are provided by a third-party and are governed by
# separate terms of service, privacy policy, and support
# documentation.

name: Upload Python Package

on:
  push:
    branches:
      - 'main'

permissions:
  contents: read

jobs:
  deploy:

    runs-on: ubuntu-latest

    steps:
    - uses: actions/checkout@v3
    - name: Set up Python
      uses: actions/setup-python@v3
      with:
        python-version: '3.x'
    - name: Install dependencies
      run: |
        python -m pip install --upgrade pip
        pip install build
    - name: Build package
      run: python -m build
    - name: Publish package
      uses: pypa/gh-action-pypi-publish@27b31702a0e7fc50959f5ad993c78deac1bdfc29
      with:
        user: __token__
        password: ${{ secrets.PYPI_API_TOKEN }}

    '''
    return content



import os
from test.test_init.kraken_testfn import kraken_testfn
from test.test_init.kraken_testfn.Class_kraken_testfn import Testfn
from test.test_init.kraken_testfn.Class_kraken_testfns import Testfns


"""
Project structure created by kraken_init

To dos
1. Configure pypi package publishing
- Todo: Configure github pypi publish action - DONE 
- Todo: Create pypi token (https://pypi.org/manage/account/)
- Todo: Add pypi api token to github secret (https://github.com/tactik8/krakentestfn/settings/secrets/actions)

2. Configure google cloud
- Todo: Create new cloud run service (https://console.cloud.google.com/run?project=kraken-v2-369412)

3. Add packages 
- Todo: Add required packages to setup.py
- Todo: Add required packages to requirements.txt
"""

def test():
    """Perform tests
    """
    os.system("pip install pytest")
    os.system("python -m pytest kraken_testfn* -vv")


#test()

#flask_routes.run_api()

testfn = Testfn()

testfns = Testfns()

    

import os

"""
Project structure created by kraken_init

To dos
1. Configure pypi package publishing
- Todo: Configure github pypi publish action - DONE 
- Todo: Create pypi token (https://pypi.org/manage/account/)
- Todo: Add pypi api token to github secret (https://github.com/tactik8/krakenfunction/settings/secrets/actions)

2. Configure google cloud
- Todo: Create new cloud run service (https://console.cloud.google.com/run?project=kraken-v2-369412)

3. Add packages 
- Todo: Add required packages to setup.py
- Todo: Add required packages to requirements.txt
"""

def test():
    """Perform tests
    """
    os.system("pip install pytest")
    os.system("python -m pytest kraken_function* -vv")


#test()

import kraken_init

#kraken_init.init('kraken_testfn', True, 'test/test_init')

from test.test_init.kraken_testfn import kraken_testfn as m

from test.test_init.kraken_testfn.class_kraken_testfn import Testfn
from test.test_init.kraken_testfn.class_kraken_testfns import Testfns

t = Testfn()

tt = Testfns()

for i in t:
    print(t._id)

tt.set(t)
tt.set(t)

for i in tt:
    print(i._id)
    


def get_filename(name):

    return f'{name}/tests/test_{name}.py'

def get_content(name, directory=None):

    class_name = name.replace('kraken_', '')
    class_name = class_name.capitalize()

    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir

    
    content = f'''
from {dir}{name}.{name} import {class_name}

def test_init():
    """
    """

    i = {class_name}()

    property = 'test_property'
    value = 'test_value'

    expected_result = 'test_value'
    
    i.set(property, value)

    result = i.get(property)

    assert result == expected_result

    '''
    return content




def get_filename(name):

    return f'{name}/__init__.py'


def get_content(name=None, directory=None):
    """
    """

    class_name = name.replace('kraken_', '')

    class_name = class_name.capitalize()
    class_name_collection = class_name + 's'

    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir

    
    content = f'''
from {dir}{name} import {name}
from {dir}{name}.class_{name} import {class_name}
from {dir}{name}.class_{name + 's'} import {class_name_collection}
    '''
    return content



def get_filename(name):

    return f'.gitignore'


def get_content(name, directory=None):
    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir

    
    content = '''
poetry.lock
pyproject.toml


    '''
    return content





def get_filename(name):

    
    return f'{name}/class_{name}.py'

def get_content(name, directory):
    """
    """
    
    class_name = name.replace('kraken_', '')
    
    class_name = class_name.capitalize()

    record_value = {}
    dir = ''
    dir = directory.replace('/', '.') + '.' if directory else dir
    
    content = f'''
    
import copy
import uuid
import os
import pkg_resources
from {dir}{name}.helpers import json
from {dir}{name} import {name} as m

"""
Notes:
To access files in data directory, use:
new_path = pkg_resources.resource_filename('{dir}{name}', old_path)

"""

class {class_name}:
    """
    The Vehicle object contains a lot of vehicles

    Args:
        arg1 (str): The arg is used for...
        arg2 (str): The arg is used for...
        arg3 (str): The arg is used for...

    Attributes:
        _objects: List of all objects initialized with this class
        record (dict): This is where we store attributes
        json (str): Record in json format
        
    """

    _objects = []    # Initialize array to store all objects instances

    def __init__(self):
        
        {class_name}._objects.append(self)    # Add itself to list of objects instances

        self._id = str(uuid.uuid4())
        self._record = {record_value}
        self._index = 0

    def __str__(self):
        """
        """
        return str(self._record)

    
    def __repr__(self):
        """
        """
        return str(self._record)

    def __iter__(self):
        """ Allows the object to be used in a for loop as a array of only itself
        """
        self._index=0
        return self

    def __next__(self):
        if self._index == 0:
            self._index = 1
            return self
        else:
            self._index = 0
            raise StopIteration


    
    def __eq__(self, other):
        """
        """
        if type(self) != type(other):
            return False
            
        if self._record == other._record:
            return True
        return False

    def __gt__(self, other):
        """
        """
        return True

        
    def set(self, property, value):
        """
        """
        self._record[property] = value
        return True

    
    def get(self, property):
        """
        """
        return self._record.get(property, None)

    
    def load(self, value):
        """
        """
        self._record = value
        return True


    def dump(self): 
        """
        """
        return copy.deepcopy(self._record)
        

    def set_json(self, value):
        """
        """
        record = json.loads(value)
        self.load(record)
        return True

    def get_json(self):
        """
        """
        return json.dumps(self.dump())

    @property
    def record(self):
        return self.dump()

    @record.setter
    def record(self, value):
        return self.load(value)
    
    @property
    def json(self):
        return self.get_json()

    @json.setter
    def json(self, value):
        return self.set_json(value)
        

    '''
    
    return content
    



    
import copy
from test.test_init.kraken_testfn.helpers import json
import os
import pkg_resources

"""
Notes:
To access files in data directory, use:
new_path = pkg_resources.resource_filename('kraken_testfn', old_path)

"""

        
def method1():
    """
    """
    
    return True


def method2():
    """
    """
    
    return True



        

    